<?php if(!defined('ABSPATH')) {die('You are not allowed to call this page directly.');}

class MpDrip {
  public function __construct() {
    add_action('mepr_display_autoresponders', array($this,'display_option_fields'));
    add_action('mepr-process-options', array($this,'store_option_fields'));
    add_action('mepr-user-signup-fields', array($this,'display_signup_field'));
    add_action('mepr-signup', array($this,'process_signup'));
    add_action('mepr-txn-store', array($this,'process_status_changes'));
    add_action('mepr-subscr-store', array($this,'process_status_changes'));
    add_action('mepr-txn-expired', array($this,'process_status_changes'), 10, 2);
    add_action('mepr-product-advanced-metabox', array($this,'display_product_override'));
    add_action('mepr-product-save-meta', array($this,'save_product_override'));
    add_filter('mepr-validate-account', array($this,'update_user_email'), 10, 2); //Need to use this hook to get old and new emails

    // Enqueue scripts
    add_action('mepr-options-admin-enqueue-script', array($this,'admin_enqueue_options_scripts'));
    add_action('mepr-product-admin-enqueue-script', array($this,'admin_enqueue_product_scripts'));

    // AJAX Endpoints
    add_action('wp_ajax_mepr_drip_ping_apikey', array($this, 'ajax_ping_apikey'));
    add_action('wp_ajax_mepr_drip_get_accounts', array($this, 'ajax_get_accounts'));
    add_action('wp_ajax_mepr_drip_get_lists',   array($this, 'ajax_get_lists'));
  }

  public function admin_enqueue_options_scripts($hook) {
    wp_register_script('mp-drip-js', MPDRIP_URL . '/drip.js');
    wp_enqueue_script('mp-drip-options-js', MPDRIP_URL.'/drip_options.js', array('mp-drip-js'));
    wp_localize_script('mp-drip-options-js', 'MeprDrip', array('wpnonce' => wp_create_nonce(MEPR_PLUGIN_SLUG)));
  }

  public function admin_enqueue_product_scripts($hook) {
    wp_register_script('mp-drip-js', MPDRIP_URL.'/drip.js');
    wp_enqueue_script('mp-drip-product-js', MPDRIP_URL.'/drip_product.js', array('mp-drip-js'));
  }

  public function update_user_email($errors, $mepr_user) {
    if( !$this->is_enabled_and_authorized() ) { return $errors; }

    //Check if the email is even changing before we do anything else
    $new_email = stripslashes($_POST['user_email']);

    if($mepr_user->user_email != $new_email) {
      //First let's update the global list_id
      $this->update_subscriber($mepr_user, $this->account_id(), $new_email);

      $products = $mepr_user->active_product_subscriptions('products');

      if(!empty($products)) {
        foreach($products as $prd) {
          $enabled = (bool)get_post_meta($prd->ID, '_meprdrip_list_override', true);
          $account_id = get_post_meta($prd->ID, '_meprdrip_account_override_id', true);

          if($enabled && !empty($account_id)) {
            $this->update_subscriber($mepr_user, $account_id, $new_email);
          }
        }
      }
    }

    return $errors;
  }

  public function update_subscriber(MeprUser $contact, $account_id, $new_email) {
    $email = $contact->user_email;

    $args = array(
      'account_id' => $account_id,
      'email' => $email,
      'new_email' => $new_email
    );

    $res = $this->call('update_subscriber', $args);

    if( $res != 'error' ) {
      return true;
    }
    else {
      return false;
    }
  }

  public function display_option_fields() {
    ?>
    <div id="mepr-drip" class="mepr-autoresponder-config">
      <input type="checkbox" name="meprdrip_enabled" id="meprdrip_enabled" <?php checked($this->is_enabled()); ?> />
      <label for="meprdrip_enabled"><?php _e('Enable Drip', 'memberpress-drip'); ?></label>
    </div>

    <div id="drip_hidden_area" class="mepr-options-sub-pane">
      <div id="mepr-drip-error" class="mepr-hidden mepr-inactive"></div>
      <div id="mepr-drip-message" class="mepr-hidden mepr-active"></div>
      <div id="meprdrip-api-key">
        <label>
          <span><?php _e('Drip API Token:', 'memberpress-drip'); ?></span>
          <input type="text" name="meprdrip_api_key" id="meprdrip_api_key" value="<?php echo $this->apikey(); ?>" class="mepr-text-input form-field" size="70" />
          <span id="mepr-drip-valid" class="mepr-active mepr-hidden"></span>
          <span id="mepr-drip-invalid" class="mepr-inactive mepr-hidden"></span>
        </label>
        <div>
      <span class="description">
      <?php _e('You can find your API Token under Settings > My User Settings at getdrip.com.', 'memberpress-drip'); ?>
      </span>
        </div>
      </div>
      <br/>
      <div id="meprdrip-account">
        <label>
          <span><?php _e('Drip Account:', 'memberpress-drip'); ?></span>
          <select name="meprdrip_account_id" id="meprdrip_account_id" data-accountid="<?php echo $this->account_id(); ?>" class="mepr-text-input form-field"></select>
        </label>
        <!--<div>
          <span class="description">
            <?php /*_e('This is typically your account used in www.getdrip.com', 'memberpress'); */?>
          </span>
        </div>-->
      </div>
      <br/>
      <div id="meprdrip-options">
        <div id="meprdrip-list-id">
          <label>
            <span><?php _e('Drip Campaign:', 'memberpress-drip'); ?></span>
            <select name="meprdrip_list_id" id="meprdrip_list_id" data-listid="<?php echo $this->list_id(); ?>" class="mepr-text-input form-field"></select>
          </label>
        </div>
        <br/>
        <div id="meprdrip-double-optin">
          <label for="meprdrip_double_optin">
            <input type="checkbox" name="meprdrip_double_optin" id="meprdrip_double_optin" class="form-field" <?php checked($this->is_double_optin_enabled()); ?> />
            <span><?php _e('Enable Double Opt-in', 'memberpress-drip'); ?></span>
          </label><br/>
      <span class="description">
      <?php _e("Members will have to click a confirmation link in an email before being added to your campaign.", 'memberpress-drip'); ?>
      </span>
        </div>
        <br/>
        <div id="meprdrip-optin">
          <label>
            <input type="checkbox" name="meprdrip_optin" id="meprdrip_optin" <?php checked($this->is_optin_enabled()); ?> />
            <span><?php _e('Enable Opt-In Checkbox', 'memberpress-drip'); ?></span>
          </label>
          <div>
      <span class="description">
        <?php _e('If checked, an opt-in checkbox will appear on all of your product registration pages.', 'memberpress-drip'); ?>
      </span>
          </div>
        </div>
        <div id="meprdrip-optin-text" class="mepr-hidden mepr-options-panel">
          <label><?php _e('Signup Checkbox Label:', 'memberpress-drip'); ?>
            <input type="text" name="meprdrip_text" id="meprdrip_text" value="<?php echo $this->optin_text(); ?>" class="form-field" size="75" />
          </label>
          <div><span class="description"><?php _e('This is the text that will display on the signup page next to your mailing list opt-in checkbox.', 'memberpress-drip'); ?></span></div>
        </div>
      </div>
    </div>
  <?php
  }

  public function validate_option_fields($errors) {
    // Nothing to validate yet -- if ever
  }

  public function update_option_fields() {
    // Nothing to do yet -- if ever
  }

  public function store_option_fields() {
    update_option('meprdrip_enabled',       (isset($_POST['meprdrip_enabled'])));
    update_option('meprdrip_account_id',    (isset($_POST['meprdrip_account_id']))?stripslashes($_POST['meprdrip_account_id']):false);
    update_option('meprdrip_api_key',       stripslashes($_POST['meprdrip_api_key']));
    update_option('meprdrip_list_id',       (isset($_POST['meprdrip_list_id']))?stripslashes($_POST['meprdrip_list_id']):false);
    update_option('meprdrip_double_optin',  (isset($_POST['meprdrip_double_optin'])));
    update_option('meprdrip_optin',         (isset($_POST['meprdrip_optin'])));
    update_option('meprdrip_text',          stripslashes($_POST['meprdrip_text']));
  }

  public function display_signup_field() {
    $mepr_options = MeprOptions::fetch();
    $post = MeprUtils::get_current_post();
    $prd = MeprProduct::is_product_page($post);

    //If the per product list is enabled, and the global list is disabled -- then we should be sure the member doesn't see this
    if($prd !== false) {
      $enabled = (bool)get_post_meta($prd->ID, '_meprdrip_list_override', true);

      if($enabled && $mepr_options->disable_global_autoresponder_list) { return; }
    }

    if($this->is_enabled_and_authorized() and $this->is_optin_enabled()) {
      $optin = (MeprUtils::is_post_request())?isset($_POST['meprdrip_opt_in']):$mepr_options->opt_in_checked_by_default;

      ?>
      <div class="mp-form-row">
        <div class="mepr-drip-signup-field">
          <div id="mepr-drip-checkbox">
            <input type="checkbox" name="meprdrip_opt_in" id="meprdrip_opt_in" class="mepr-form-checkbox" <?php checked($optin); ?> />
            <span class="mepr-drip-message"><?php echo $this->optin_text(); ?></span>
          </div>
          <div id="mepr-drip-privacy">
            <small>
              <a href="http://www.getdrip.com/terms/" class="mepr-drip-privacy-link" target="_blank"><?php _e('We Respect Your Privacy', 'memberpress-drip'); ?></a>
            </small>
          </div>
        </div>
      </div>
    <?php
    }
  }

  public function process_signup($txn) {
    $mepr_options = MeprOptions::fetch();

    $usr = $txn->user();
    $prd = $txn->product();

    $enabled = (bool)get_post_meta($prd->ID, '_meprdrip_list_override', true);

    //If the per product list is enabled, and the global list is disabled -- then we should be sure the member doesn't get added
    if(!$this->is_enabled_and_authorized() || ($enabled && $mepr_options->disable_global_autoresponder_list)) { return; }

    // if !is_option_enabled then opted_in is always true
    $opted_in = true;
    if($this->is_optin_enabled()) { $opted_in = isset($_POST['meprdrip_opt_in']); }

    if($opted_in) {
      $this->add_subscriber($usr, $this->account_id() , $this->list_id(), $this->is_double_optin_enabled());
    }
  }

  public function process_status_changes($obj, $sub_status = false) {
    if(!$this->is_enabled_and_authorized()) { return; }

    if($obj instanceof MeprTransaction && $sub_status !== false && $sub_status == MeprSubscription::$active_str) {
      return; //This is an expiring transaction which is part of an active subscription, so don't remove the user from the list
    }

    $usr = $obj->user();
    $prd = $obj->product();

    //Member is active so let's not remove them
    if($usr->is_active_on_membership($prd)) {
      $this->maybe_add_subscriber($obj, $usr);
    }
    else {
      $this->maybe_delete_subscriber($obj, $usr);
    }
  }

  public function maybe_add_subscriber($obj, $usr) {
    $enabled = (bool)get_post_meta($obj->product_id, '_meprdrip_list_override', true);
    $account_id = get_post_meta($obj->product_id, '_meprdrip_account_override_id', true);
    $list_id = get_post_meta($obj->product_id, '_meprdrip_list_override_id', true);

    if($enabled && !empty($list_id) && $this->is_enabled_and_authorized()) {
      return $this->add_subscriber($usr, $account_id, $list_id);
    }

    return false;
  }

  public function maybe_delete_subscriber($obj, $usr) {
    $enabled = (bool)get_post_meta($obj->product_id, '_meprdrip_list_override', true);
    $account_id = get_post_meta($obj->product_id, '_meprdrip_account_override_id', true);
    $list_id = get_post_meta($obj->product_id, '_meprdrip_list_override_id', true);

    if($enabled && !empty($list_id) && $this->is_enabled_and_authorized()) {
      return $this->delete_subscriber($usr, $account_id, $list_id);
    }

    return false;
  }

  public function validate_signup_field($errors) {
    // Nothing to validate -- if ever
  }

  public function display_product_override($product) {
    if(!$this->is_enabled_and_authorized()) { return; }

    $override_list = (bool)get_post_meta($product->ID, '_meprdrip_list_override', true);
    $override_account_id = get_post_meta($product->ID, '_meprdrip_account_override_id', true);
    $override_list_id = get_post_meta($product->ID, '_meprdrip_list_override_id', true);

    ?>
    <div id="mepr-drip" class="mepr-product-adv-item">
      <input type="checkbox" name="meprdrip_list_override" id="meprdrip_list_override" data-accountid="<?php echo $this->account_id(); ?>" data-apikey="<?php echo $this->apikey(); ?>" <?php checked($override_list); ?> />
      <label for="meprdrip_list_override"><?php _e('Get Drip Campaign for this Product', 'memberpress-drip'); ?></label>

      <?php MeprAppHelper::info_tooltip('meprdrip-list-override',
        __('Enable Product Drip campaign', 'memberpress-drip'),
        __('If this is set the member will be added to this list when their payment is completed for this product. If the member cancels or you refund their subscription, they will be removed from the list automatically. You must have your Drip API Token set in the Options before this will work.', 'memberpress-drip'));
      ?>

      <div id="meprdrip_override_area" class="mepr-hidden product-options-panel">
        <label><?php _e('Drip Account: ', 'memberpress-drip'); ?></label>
        <select name="meprdrip_account_override_id" id="meprdrip_account_override_id" data-accountid="<?php echo stripslashes($override_account_id); ?>" class="mepr-text-input form-field"></select>
        <br/>
        <label><?php _e('Drip Campaign: ', 'memberpress-drip'); ?></label>
        <select name="meprdrip_list_override_id" id="meprdrip_list_override_id" data-listid="<?php echo stripslashes($override_list_id); ?>" class="mepr-text-input form-field"></select>
      </div>
    </div>
  <?php
  }

  public function save_product_override($product) {
    if(!$this->is_enabled_and_authorized()) { return; }

    if(isset($_POST['meprdrip_list_override'])) {
      update_post_meta($product->ID, '_meprdrip_list_override', true);
      update_post_meta($product->ID, '_meprdrip_account_override_id', stripslashes($_POST['meprdrip_account_override_id']));
      update_post_meta($product->ID, '_meprdrip_list_override_id', stripslashes($_POST['meprdrip_list_override_id']));
    }
    else {
      update_post_meta($product->ID, '_meprdrip_list_override', false);
    }
  }

  public function ajax_ping_apikey() {
    $this->ajax_get_accounts();
  }

  public function ajax_get_accounts() {
    $args = array();

    // Validate nonce and user capabilities
    if(!isset($_POST['wpnonce']) || !wp_verify_nonce($_POST['wpnonce'], MEPR_PLUGIN_SLUG) || !MeprUtils::is_mepr_admin()) {
      die(json_encode(array('error' => __('Hey yo, why you creepin\'?', 'memberpress-drip'), 'type' => 'memberpress')));
    }

    // Validate inputs
    if(!isset($_POST['apikey'])) {
      die(json_encode(array('error' => __('No apikey code was sent', 'memberpress-drip'), 'type' => 'memberpress')));
    }

    die($this->call('get_accounts', $args, $_POST['apikey']));
  }

  public function ajax_get_lists() {
    // Validate nonce and user capabilities
    if(!isset($_POST['wpnonce']) || !wp_verify_nonce($_POST['wpnonce'], MEPR_PLUGIN_SLUG) || !MeprUtils::is_mepr_admin()) {
      die(json_encode(array('error' => __('Hey yo, why you creepin\'?', 'memberpress-drip'), 'type' => 'memberpress')));
    }

    // Validate inputs
    if(!isset($_POST['apikey']) || !isset($_POST['account'])) {
      die(json_encode(array('error' => __('No apikey code was sent', 'memberpress-drip'), 'type' => 'memberpress')));
    }

    $args = array( "account_id"=>$_POST['account'] );

    die($this->call('get_campaigns', $args, $_POST['apikey']));
  }

   public function add_subscriber(MeprUser $contact, $account_id, $list_id, $double_optin=false) {
     if( $double_optin == '' ){ $double_optin = false; }

     $args = array(
       'account_id' => $account_id,
       'campaign_id' => $list_id ,
       'email' => $contact->user_email,
       'double_optin' => $double_optin,
       'custom_fields' => array(
         'name' => $contact->first_name . ' ' . $contact->last_name
       )
     );

     $res = $this->call('add_subscriber', $args);

     return ($res != 'error');
  }

  /* unsubscribe */
  public function delete_subscriber(MeprUser $contact, $account_id, $list_id ) {
    $email = $contact->user_email;

    $args = array(
      'account_id' => $account_id,
      'email' => $email,
      "campaign_id" => $list_id
    );

    $res = $this->call('unsubscribe_subscriber', $args);

    return ($res != 'error');
  }

  private function call($endpoint, $args=array(), $apikey=null) {
    if( is_null($apikey) ) { $apikey = $this->apikey(); }

    require_once(MPDRIP_PATH.'/Drip_API.class.php');
    $api = new Drip_Api($apikey);

    if( $endpoint == 'get_campaigns' ){
      $response = $api->get_campaigns($args);
    }
    elseif ( $endpoint == 'add_subscriber' ){
      $response = $api->subscribe_subscriber($args);
    }
    elseif ( $endpoint == 'unsubscribe_subscriber' ){
      $response = $api->unsubscribe_subscriber($args);
    }
    elseif( $endpoint == 'update_subscriber') {
       $response = $api->create_or_update_subscriber($args);
    }
    else {
      $response = $api->$endpoint();
    }

    if( $response ) {
      return json_encode($response);
    }
    else{
      return 'error';
    }
  }

  // I realize these are more like model methods
  // but we want everything centralized here people
  private function is_enabled() {
    return get_option('meprdrip_enabled', false);
  }

  private function is_authorized() {
    $apikey = get_option('meprdrip_api_key', '');
    return !empty($apikey);
  }

  private function is_enabled_and_authorized() {
    return ($this->is_enabled() and $this->is_authorized());
  }

  private function account_id() {
    return get_option('meprdrip_account_id', '');
  }

  private function apikey() {
    return get_option('meprdrip_api_key', '');
  }

  private function list_id() {
    return get_option('meprdrip_list_id', false);
  }

  private function is_double_optin_enabled() {
    return get_option('meprdrip_double_optin', true);
  }

  private function is_optin_enabled() {
    return get_option('meprdrip_optin', true);
  }

  private function optin_text() {
    $default = sprintf(__('Sign Up for the %s Newsletter', 'memberpress-drip'), get_option('blogname'));
    return get_option('meprdrip_text', $default);
  }

} //END CLASS


