<?php
if(!defined('ABSPATH')) {die('You are not allowed to call this page directly.');}
/*
Integration of WPSimplePay into MemberPress
*/
class MpWPSimplePay {
  public function __construct() {
    // ADMIN UI
    add_filter('simpay_form_settings_meta_tabs_li', array($this, 'add_membership_product_tab'), 10, 2);
    add_action('simpay_form_settings_meta_options_panel', array($this, 'show_membership_product_tab_content'));
    add_action('simpay_save_form_settings', array($this, 'save_product_tab_content'));

    // CHECKOUT
    add_action( 'simpay_charge_created', array($this,'charge_created') );
    add_action( 'simpay_subscription_created', array($this,'subscription_created'), 10, 2 );
  }

  // Add a new tab when editing the WPSimplePay Forms for the Membership syncing settings
  public function add_membership_product_tab($tabs) {
    global $post;

    if(!isset($post->ID)) { return; }

    $tabs['mepr'] = array(
      'label'  => __('MemberPress', 'memberpress-wpsimplepay'),
      'target' => 'mepr_membership',
      'class'  => array(),
      'icon'   => ''
    );

    return $tabs;
  }

  //Show the options on the MemberPress tab when editing a WPSimplePay Form
  public function show_membership_product_tab_content($form_id) {
    if(!isset($form_id)) { return; }

    $selected = $this->get_chosen_membership($form_id);
    $checked = $this->get_sync_when_processing($form_id);
    $all_memberships = MeprCptModel::all('MeprProduct');

    if(empty($all_memberships)) {
      ?>
        <div id="mepr_membership" class="simpay-panel simpay-panel-hidden">
          <?php _e('No Memberships Created Yet', 'memberpress-wpsimplepay'); ?>
        </div>
      <?php

      return;
    }

    ?>
      <div id="mepr_membership" class="simpay-panel simpay-panel-hidden">
        <table>
          <thead>
            <tr>
              <th colspan="2"><?php _e('Auto-Create MemberPress Membership Level', 'memberpress-wpsimplepay'); ?></th>
            </tr>
          </thead>
          <tbody class="simpay-panel-section">
            <tr class="simpay-panel-field">
              <th>
                <label for="mepr_membership_id"><?php _e('Membership', 'memberpress-wpsimplepay'); ?></label>
              </th>
              <td>
                <select name="mepr_membership_id" id="mepr_membership_id" class="simpay-field-select">
                  <option value="0"><?php _e('None (Not Synced)', 'memberpress-wpsimplepay'); ?></option>
                  <?php foreach($all_memberships as $membership): ?>
                    <option value="<?php echo $membership->ID; ?>" <?php selected($selected,$membership->ID); ?>><?php echo $membership->post_title; ?></option>
                  <?php endforeach; ?>
                </select>
                <p class="description"><?php _e('Select a Membership level to add the user to when they purchase using this form. Note that when a membership is chosen WP Simple Pay will automatically display user registration form fields', 'memberpress-wpsimplepay'); ?></p>
              </td>
            </tr>
            <tr class="simpay-panel-field">
              <th scope="row">
                <label for="mepr_sync_when_processing"><?php _e('Access Before Payment', 'memberpress-wpsimplepay'); ?></label>
              </th>
              <td>
                <input type="checkbox" name="mepr_sync_when_processing" <?php checked($checked) ?> />
                <p class="description"><?php _e('When checked, this will allow the user to have access to the membership before the payment is recorded.', 'memberpress-wpsimplepay'); ?></p>
              </td>
            </tr>
          </tbody>
        </table>
      </div>
    <?php
  }

  //Save the Membership options when updating WPSimplePay Product
  public function save_product_tab_content($form_id) {
    if( isset($_POST['mepr_membership_id']) && !empty($_POST['mepr_membership_id']) && (int)$_POST['mepr_membership_id'] > 0) {
      update_post_meta($form_id, '_mepr_membership_id', (int)$_POST['mepr_membership_id']);

      if(isset($_POST['mepr_sync_when_processing'])) {
        update_post_meta($form_id, '_mepr_sync_when_processing', true);
      } else {
        update_post_meta($form_id, '_mepr_sync_when_processing', false);
      }
    }
    else {
      update_post_meta($form_id, '_mepr_membership_id', 0);
      update_post_meta($form_id, '_mepr_sync_when_processing', false);
    }
  }

  public function charge_created($charge) {
    global $simpay_form;

    MeprUtils::debug_log('****** Charge Created -> $simpay_form->id: '.$simpay_form->id);

    $user_id = MpWPSimplePayUtils::create_user_from_stripe( $charge );

    $membership_id = $this->get_chosen_membership( $simpay_form->id );
    $txn = MpWPSimplePayUtils::create_transaction_from_stripe( $user_id, $membership_id, $charge );

    MeprUtils::send_signup_notices($txn);
  }

  public function subscription_created($charge, $customer) {
    global $simpay_form;

    MeprUtils::debug_log('****** Subscription Created -> $simpay_form->id: '.$simpay_form->id);

    $user_id = MpWPSimplePayUtils::create_user_from_stripe( $charge, $customer );

    $membership_id = $this->get_chosen_membership( $simpay_form->id );
    $sub = MpWPSimplePayUtils::create_subscription_from_stripe( $simpay_form->id, $user_id, $membership_id, $charge, $customer );
    $first_txn = $sub->first_txn();

    MeprUtils::send_signup_notices($first_txn);
  }

  //Which Membership are we syncing with?
  private function get_chosen_membership($form_id) {
    if(!$form_id) { return 0; }

    return get_post_meta($form_id, '_mepr_membership_id', true);
  }

  //Should we create the Membership when the Order status is "processing"?
  private function get_sync_when_processing($form_id) {
    if(!$form_id) { return false; }

    return get_post_meta($form_id, '_mepr_sync_when_processing', true);
  }

} //End class

